@@ -1,5 +1,4 @@
 import json
-import pycurl
 import time
 import urllib
 from collections import OrderedDict
@@ -47,14 +46,19 @@ class Instagram:
         self.isLoggedIn = False  # // Session status
         self.rank_token = None  # // Rank token
         self.IGDataPath = None  # // Data storage path
+        self.customPath = False
         self.http = None
         self.settings = None
+        self.proxy = None  # Full Proxy
+        self.proxyHost = None  # Proxy Host and Port
+        self.proxyAuth = None  # Proxy User and Pass
 
         self.debug = debug
         self.device_id = SignatureUtils.generateDeviceId(hashlib.md5(username + password))
 
         if IGDataPath is not None:
             self.IGDataPath = IGDataPath
+            self.customPath = True
         else:
             self.IGDataPath = os.path.join(
                 os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data'),
@@ -94,8 +98,19 @@ class Instagram:
             self.username_id = self.settings.get('username_id')
             self.rank_token = self.username_id + '_' + self.uuid
             self.token = self.settings.get('token')
+        else:
+            self.isLoggedIn = False
+
+    def checkSettings(self, username):
+        if self.customPath:
+            self.IGDataPath = os.path.join(
+                os.path.join(os.path.dirname(os.path.realpath(__file__)), 'data'),
+                username,
+                ''
+            )
+
+        if not os.path.isdir(self.IGDataPath): os.mkdir(self.IGDataPath, 0777)
 
-    def checkSettings(self,username):
         self.settings = Settings(
             os.path.join(self.IGDataPath, 'settings-' + username + '.dat')
         )
@@ -103,12 +118,50 @@ class Instagram:
             self.settings.set('version', Constants.VERSION)
 
         if (self.settings.get('user_agent') is None) or (
-            LooseVersion(self.settings.get('version')) < LooseVersion(Constants.VERSION)):
+                    LooseVersion(self.settings.get('version')) < LooseVersion(Constants.VERSION)):
             userAgent = UserAgent(self)
             ua = userAgent.buildUserAgent()
             self.settings.set('version', Constants.VERSION)
             self.settings.set('user_agent', ua)
 
+    def setProxy(self, proxy, port=None, username=None, password=None):
+        """
+        Set the proxy.
+
+        :type proxy: str
+        :param proxy: Full proxy string. Ex: user:pass@192.168.0.0:8080
+                        Use $proxy = "" to clear proxy
+        :type port: int
+        :param port: Port of proxy
+        :type username: str
+        :param username: Username for proxy
+        :type password: str
+        :param password: Password for proxy
+
+        :raises: InstagramException
+        """
+        self.proxy = proxy
+
+        if proxy == '':
+            return
+
+        proxy = parse_url(proxy)
+
+        if port and isinstance(port, int):
+            proxy['port'] = int(port)
+
+        if username and password:
+            proxy['user'] = username
+            proxy['pass'] = password
+
+        if proxy['host'] and proxy['port'] and isinstance(proxy['port'], int):
+            self.proxyHost = proxy['host'] + ':' + proxy['port']
+        else:
+            raise InstagramException('Proxy host error. Please check ip address and port of proxy.')
+
+        if proxy['user'] and proxy['pass']:
+            self.proxyAuth = proxy['user'] + ':' + proxy['pass']
+
     def login(self, force=False):
         """
         Login to Instagram.
@@ -120,15 +173,19 @@ class Instagram:
         """
         if (not self.isLoggedIn) or force:
             fetch = self.http.request(
-                'si/fetch_headers/?challenge_type=signup&guid=' + SignatureUtils.generateUUID(False), None,True)
+                'si/fetch_headers/?challenge_type=signup&guid=' + SignatureUtils.generateUUID(False), None, True)
+            header = fetch[0]
+            response = ChallengeResponse(fetch[1])
 
-            if fetch[0] == '':
+            if not header or not response.isOk():
                 raise InstagramException("Couldn't get challenge, check your connection")
-                #return response #FIXME unreachable code
+                # return response #FIXME unreachable code
 
             match = re.search(r'^Set-Cookie: csrftoken=([^;]+)', fetch[0], re.MULTILINE)
             if match:
                 self.token = match.group(1)
+            else:
+                raise InstagramException('Missing csfrtoken')
 
             data = OrderedDict([
                 ('username', self.username),
@@ -187,6 +244,30 @@ class Instagram:
     def megaphoneLog(self):
         return self.http.request('megaphone/log/')[1]
 
+    def getPendingInbox(self):
+        """
+        Pending Inbox
+
+        :rtype: object
+        :return: Pending Inbox Data
+        """
+        pendingInbox = self.request('direct_v2/pending_inbox/?')[1]
+
+        if pendingInbox['status'] != 'ok':
+            raise InstagramException(pendingInbox['message'] + "\n")
+            # return FIXME unreachable code
+
+        return pendingInbox
+
+    def explore(self):
+        """
+        Explore Tab
+
+        :rtype: object
+        :return: Explore data
+        """
+        return self.request('discover/explore/?')[1]
+
     def expose(self):
         data = json.dumps(
             OrderedDict([
@@ -238,6 +319,44 @@ class Instagram:
     def direct_share(self, media_id, recipients, text=None):
         self.http.direct_share(media_id, recipients, text)
 
+    def directThread(self, threadId):
+        """
+        Direct Thread Data
+
+        :type threadId: str
+        :param threadId: Thread Id
+        :rtype: object
+        :return: Direct Thread Data
+        """
+        directThread = self.request("direct_v2/threads/" + str(threadId) + "/?")[1]
+
+        if directThread['status'] != 'ok':
+            raise InstagramException(directThread['message'] + "\n")
+            # return Fixme unreachable code
+
+        return directThread
+
+    def directThreadAction(self, threadId, threadAction):
+        """
+        Direct Thread Action
+
+        :type threadId: str
+        :param threadId: Thread Id
+        :type threadAction: str
+        :param threadAction: Thread Action 'approve' OR 'decline' OR 'block'
+        :rtype: object
+        :return: Direct Thread Action Data
+        """
+        data = json.dumps(
+            OrderedDict([
+                ('_uuid', self.uuid),
+                ('_uid', self.username_id),
+                ('_csrftoken', self.token)
+            ])
+        )
+        return self.request("direct_v2/threads/" + str(threadId) + "/" + str(threadAction) + "/",
+                            self.generateSignature(data))[1]
+
     def configureVideo(self, upload_id, video, caption=''):
 
         self.uploadPhoto(video, caption, upload_id)
@@ -414,7 +533,7 @@ class Instagram:
         )
         return self.http.request("media/" + mediaId + "/comment/", SignatureUtils.generateSignature(data))[1]
 
-    def deleteComment(self, mediaId, captionText, commentId):
+    def deleteComment(self, mediaId, commentId):
         """
         Delete Comment.
         :type mediaId: str
@@ -428,14 +547,43 @@ class Instagram:
             OrderedDict([
                 ('_uuid', self.uuid),
                 ('_uid', self.username_id),
-                ('_csrftoken', self.token),
-                ('caption_text', captionText)  # BUG!!!
+                ('_csrftoken', self.token)
             ])
         )
         return \
             self.http.request("media/" + mediaId + "/comment/" + commentId + "/delete/",
-                              SignatureUtils.generateSignature(data))[
-                1]
+                              SignatureUtils.generateSignature(data))[1]
+
+    def deleteCommentsBulk(self, mediaId, commentIds):
+        """
+        Delete Comment Bulk.
+
+        :type mediaId: str
+        :param mediaId: Media ID
+        :type commentIds: list
+        :param commentIds: List of comments to delete
+        :rtype: object
+        :return: Delete Comment Bulk Data
+        """
+        if not isinstance(commentIds, list):
+            commentIds = [commentIds]
+
+        string = []
+        for commentId in commentIds:
+            string.append(str(commentId))
+
+        comment_ids_to_delete = ','.join(string)
+
+        data = json.dumps(
+            OrderedDict([
+                ('_uuid', self.uuid),
+                ('_uid', self.username_id),
+                ('_csrftoken', self.token),
+                ('comment_ids_to_delete', comment_ids_to_delete)
+            ])
+        )
+        return self.http.request("media/" + mediaId + "/comment/bulk_delete/",
+                                 SignatureUtils.generateSignature(data))[1]
 
     def changeProfilePicture(self, photo):
         """
@@ -535,6 +683,30 @@ class Instagram:
 
         return self.http.request('accounts/edit_profile/', SignatureUtils.generateSignature(data))[1]
 
+    def changePassword(self, oldPassword, newPassword):
+        """
+        Change Password.
+
+        :type oldPassword: str
+        :param oldPassword: Old Password
+        :type newPassword: str
+        :param newPassword: New Password
+        :rtype: object
+        :return: Change Password Data
+        """
+
+        data = json.dumps(
+            OrderedDict([
+                ('_uuid', self.uuid),
+                ('_uid', self.username_id),
+                ('_csrftoken', self.token),
+                ('old_password', oldPassword),
+                ('new_password1', newPassword),
+                ('new_password2', newPassword)
+            ])
+        )
+        return self.http.request('accounts/change_password/', SignatureUtils.generateSignature(data))[1]
+
     def getUsernameInfo(self, usernameId):
         """
         Get username info.
@@ -637,9 +809,10 @@ class Instagram:
         :rtype: object
         :return:
         """
-        likers = self.http.request("media/" + mediaId + "/likers/?")[1]
-        if likers['status'] != 'ok':
-            raise InstagramException(likers['message'] + "\n")
+        likers = MediaLikersResponse(self.http.request("media/" + mediaId + "/likers/")[1])
+        if not likers.isOk():
+            raise InstagramException(likers['message'] + "\n")  # bug is no longer json
+            # return #fixme unreachable code
 
         return likers
 
@@ -666,6 +839,16 @@ class Instagram:
         """
         return self.getGeoMedia(self.username_id)
 
+    def searchLocation(self, latitude, longitude):
+        locations = LocationResponse(self.http.request(
+            "location_search/?rank_token=" + self.rank_token + "&latitude=" + latitude + "&longitude=" + longitude)[1])
+
+        if not locations.isOk():
+            raise InstagramException(locations.getMessage() + "\n")
+            # return fixme unreachable code
+
+        return locations
+
     def fbUserSearch(self, query):
         """
         facebook user search.
@@ -711,13 +894,16 @@ class Instagram:
         :rtype: object
         :return: query data
         """
-        query = self.http.request("users/" + usernameName + "/usernameinfo/")[1]
+        query = UsernameInfoResponse(self.http.request("users/" + usernameName + "/usernameinfo/")[1])
 
-        if query['status'] != 'ok':
-            raise InstagramException(query['message'] + "\n")
+        if not query.isOk():
+            raise InstagramException(query.getMessage() + "\n")
 
         return query
 
+    def getUsernameId(self, username):
+        return self.searchUsername(username).getUsernameId()
+
     def syncFromAdressBook(self, contacts):
         """
         Search users using addres book.
@@ -805,7 +991,7 @@ class Instagram:
 
         return hashtagFeed
 
-    def searchLocation(self, query):
+    def searchFBLocation(self, query):
         """
         Get locations.
         :type query: str
@@ -875,9 +1061,9 @@ class Instagram:
         :rtype: object
         :return: followers data
         """
-        return self.http.request(
+        return FollowingResponse(self.http.request(
             "friendships/" + usernameId + "/following/?max_id=" + maxid + "&ig_sig_key_version=" \
-            + Constants.SIG_KEY_VERSION + "&rank_token=" + self.rank_token)[1]
+            + Constants.SIG_KEY_VERSION + "&rank_token=" + self.rank_token)[1])
 
     def getUserFollowers(self, usernameId, maxid=''):
         """
@@ -888,9 +1074,9 @@ class Instagram:
         :rtype: object
         :return: followers data
         """
-        return self.http.request(
+        return FollowerResponse(self.http.request(
             "friendships/" + usernameId + "/followers/?max_id=" + maxid \
-            + "&ig_sig_key_version=" + Constants.SIG_KEY_VERSION + "&rank_token=" + self.rank_token)[1]
+            + "&ig_sig_key_version=" + Constants.SIG_KEY_VERSION + "&rank_token=" + self.rank_token)[1])
 
     def getSelfUserFollowers(self):
         """
@@ -901,15 +1087,14 @@ class Instagram:
         """
         return self.getUserFollowers(self.username_id)
 
-    def getUsersFollowing(self):
+    def getSelfUsersFollowing(self):
         """
         Get self users we are following.
 
         :rtype: object
         :return: users we are following data
         """
-        return self.http.request('friendships/following/?ig_sig_key_version=' \
-                                 + Constants.SIG_KEY_VERSION + "&rank_token=" + self.rank_token)[1]
+        return self.getUserFollowings(self.username_id)
 
     def like(self, mediaId):
         """
@@ -1124,14 +1309,8 @@ class Instagram:
         endpoint = 'feed/liked/?' + (('max_id=' + str(maxid) + '&') if maxid is not None else '')
         return self.http.request(endpoint)[1]
 
-    def setProxy(self, proxy):
-        self.http.setProxy(proxy)
+    def verifyPeer(self, enable):
+        self.http.verifyPeer(enable)
 
-    def setProxyPort(self, port):
-        self.http.setProxyPort(port)
-
-    def setProxyCredentials(self, username, password):
-        self.http.setProxyCredentials(username, password)
-
-    def clearProxy(self):
-        self.http.clearProxy()
\ No newline at end of file
+    def verifyHost(self, enable):
+        self.http.verifyHost(enable)
