@@ -1,3 +1,4 @@
+from Checkpoint import Checkpoint
 from Constants import Constants
 from Instagram import Instagram
 from InstagramException import InstagramException
@@ -6,4 +7,4 @@ from SignatureUtils import SignatureUtils
 from Utils import *
 from http import *
 
-__all__ = ["Constants", "Instagram", "InstagramException", "InstagramRegistration", "http", "Utils"]
+__all__ = ["Constants", "Instagram", "Checkpoint", "InstagramException", "InstagramRegistration", "http", "Utils"]
