@@ -1,11 +1,12 @@
 import hmac
 import json
-import locale
 import math
 import pycurl
 import time
 from collections import OrderedDict
 
+import locale
+
 try:
     from io import BytesIO
 except ImportError:
@@ -18,14 +19,15 @@ from InstagramAPI.src.Utils import *
 from InstagramAPI.src.http.Response import *
 
 
-class HttpInterface:
+class HttpInterface(object):
     def __init__(self, parent):
+        self.parent = None
+        self.userAgent = None
+        self.verifyPeer = False
+        self.verifyHost = False
+
         self.parent = parent
         self.userAgent = self.parent.settings.get('user_agent')
-        self.proxy = None
-        self.proxyPort = None
-        self.proxyUser = None
-        self.proxyPass = None
 
     def request(self, endpoint, post=None, login=False):
         buffer = BytesIO()
@@ -49,23 +51,20 @@ class HttpInterface:
         ch.setopt(pycurl.HEADER, True)
         ch.setopt(pycurl.HTTPHEADER, headers)
         ch.setopt(pycurl.VERBOSE, False)
-        ch.setopt(pycurl.SSL_VERIFYPEER, False)
-        ch.setopt(pycurl.SSL_VERIFYHOST, False)
+        ch.setopt(pycurl.SSL_VERIFYPEER, self.verifyPeer)
+        ch.setopt(pycurl.SSL_VERIFYHOST, self.verifyHost)
         ch.setopt(pycurl.COOKIEFILE, self.parent.IGDataPath + self.parent.username + '-cookies.dat')
         ch.setopt(pycurl.COOKIEJAR, self.parent.IGDataPath + self.parent.username + '-cookies.dat')
 
-        if not self.proxy is None:
-            ch.setopt(pycurl.PROXYPORT, self.proxyPort)
-            ch.setopt(pycurl.PROXYTYPE, 'HTTP')
-            ch.setopt(pycurl.PROXY, self.proxy)
-
-        if not self.proxyUser is None:
-            ch.setopt(pycurl.PROXYUSERPWD, self.proxyUser + ':' + self.proxyPass)
-
         if post:
             ch.setopt(pycurl.POST, True)
             ch.setopt(pycurl.POSTFIELDS, post)
 
+        if self.parent.proxy:
+            ch.setopt(pycurl.PROXY, self.parent.proxyHost)
+            if self.parent.proxyAuth:
+                ch.setopt(pycurl.PROXYUSERPWD, self.parent.proxyAuth)
+
         ch.perform()
         resp = buffer.getvalue()
         header_len = ch.getinfo(pycurl.HEADER_SIZE)
@@ -159,21 +158,18 @@ class HttpInterface:
         ch.setopt(pycurl.FOLLOWLOCATION, True)
         ch.setopt(pycurl.HEADER, True)
         ch.setopt(pycurl.VERBOSE, self.parent.debug)
-        ch.setopt(pycurl.SSL_VERIFYPEER, False)
-        ch.setopt(pycurl.SSL_VERIFYHOST, False)
+        ch.setopt(pycurl.SSL_VERIFYPEER, self.verifyPeer)
+        ch.setopt(pycurl.SSL_VERIFYHOST, self.verifyHost)
         ch.setopt(pycurl.HTTPHEADER, headers)
         ch.setopt(pycurl.COOKIEFILE, self.parent.IGDataPath + self.parent.username + "-cookies.dat")
         ch.setopt(pycurl.COOKIEJAR, self.parent.IGDataPath + self.parent.username + "-cookies.dat")
         ch.setopt(pycurl.POST, True)
         ch.setopt(pycurl.POSTFIELDS, data)
 
-        if not self.proxy is None:
-            ch.setopt(pycurl.PROXYPORT, self.proxyPort)
-            ch.setopt(pycurl.PROXYTYPE, 'HTTP')
-            ch.setopt(pycurl.PROXY, self.proxy)
-
-        if not self.proxyUser is None:
-            ch.setopt(pycurl.PROXYUSERPWD, self.proxyUser + ':' + self.proxyPass)
+        if self.parent.proxy:
+            ch.setopt(pycurl.PROXY, self.parent.proxyHost)
+            if self.parent.proxyAuth:
+                ch.setopt(pycurl.PROXYUSERPWD, self.parent.proxyAuth)
 
         ch.perform()
         resp = buffer.getvalue()
@@ -191,6 +187,10 @@ class HttpInterface:
             print 'RESPONSE: ' + resp[header_len:] + "\n"
 
         configure = self.parent.configure(upload.getUploadId(), photo, caption)
+
+        if not configure.isOk():
+            raise InstagramException(configure.getMessage())
+
         self.parent.expose()
 
         return configure
@@ -250,13 +250,10 @@ class HttpInterface:
         ch.setopt(pycurl.POST, True)
         ch.setopt(pycurl.POSTFIELDS, data)
 
-        if not self.proxy is None:
-            ch.setopt(pycurl.PROXYPORT, self.proxyPort)
-            ch.setopt(pycurl.PROXYTYPE, 'HTTP')
-            ch.setopt(pycurl.PROXY, self.proxy)
-
-        if not self.proxyUser is None:
-            ch.setopt(pycurl.PROXYUSERPWD, self.proxyUser + ':' + self.proxyPass)
+        if self.parent.proxy:
+            ch.setopt(pycurl.PROXY, self.parent.proxyHost)
+            if self.parent.proxyAuth:
+                ch.setopt(pycurl.PROXYUSERPWD, self.parent.proxyAuth)
 
         ch.perform()
         resp = buffer.getvalue()
@@ -306,6 +303,11 @@ class HttpInterface:
             ch.setopt(pycurl.POST, True)
             ch.setopt(pycurl.POSTFIELDS, videoData[start:end])
 
+            if self.parent.proxy:
+                ch.setopt(pycurl.PROXY, self.parent.proxyHost)
+                if self.parent.proxyAuth:
+                    ch.setopt(pycurl.PROXYUSERPWD, self.parent.proxyAuth)
+
             ch.perform()
             result = buffer.getvalue()
             header_len = ch.getinfo(pycurl.HEADER_SIZE)
@@ -388,21 +390,18 @@ class HttpInterface:
         ch.setopt(pycurl.FOLLOWLOCATION, True)
         ch.setopt(pycurl.HEADER, True)
         ch.setopt(pycurl.VERBOSE, self.parent.debug)
-        ch.setopt(pycurl.SSL_VERIFYPEER, False)
-        ch.setopt(pycurl.SSL_VERIFYHOST, False)
+        ch.setopt(pycurl.SSL_VERIFYPEER, self.verifyPeer)
+        ch.setopt(pycurl.SSL_VERIFYHOST, self.verifyHost)
         ch.setopt(pycurl.HTTPHEADER, headers)
         ch.setopt(pycurl.COOKIEFILE, self.parent.IGDataPath + self.parent.username + "-cookies.dat")
         ch.setopt(pycurl.COOKIEJAR, self.parent.IGDataPath + self.parent.username + "-cookies.dat")
         ch.setopt(pycurl.POST, True)
         ch.setopt(pycurl.POSTFIELDS, data)
 
-        if not self.proxy is None:
-            ch.setopt(pycurl.PROXYPORT, self.proxyPort)
-            ch.setopt(pycurl.PROXYTYPE, 'HTTP')
-            ch.setopt(pycurl.PROXY, self.proxy)
-
-        if not self.proxyUser is None:
-            ch.setopt(pycurl.PROXYUSERPWD, self.proxyUser + ':' + self.proxyPass)
+        if self.parent.proxy:
+            ch.setopt(pycurl.PROXY, self.parent.proxyHost)
+            if self.parent.proxyAuth:
+                ch.setopt(pycurl.PROXYUSERPWD, self.parent.proxyAuth)
 
         ch.perform()
 
@@ -474,21 +473,18 @@ class HttpInterface:
         ch.setopt(pycurl.FOLLOWLOCATION, True)
         ch.setopt(pycurl.HEADER, True)
         ch.setopt(pycurl.VERBOSE, self.parent.debug)
-        ch.setopt(pycurl.SSL_VERIFYPEER, False)
-        ch.setopt(pycurl.SSL_VERIFYHOST, False)
+        ch.setopt(pycurl.SSL_VERIFYPEER, self.verifyPeer)
+        ch.setopt(pycurl.SSL_VERIFYHOST, self.verifyHost)
         ch.setopt(pycurl.HTTPHEADER, headers)
         ch.setopt(pycurl.COOKIEFILE, self.parent.IGDataPath + self.parent.username + "-cookies.dat")
         ch.setopt(pycurl.COOKIEJAR, self.parent.IGDataPath + self.parent.username + "-cookies.dat")
         ch.setopt(pycurl.POST, True)
         ch.setopt(pycurl.POSTFIELDS, data)
 
-        if not self.proxy is None:
-            ch.setopt(pycurl.PROXYPORT, self.proxyPort)
-            ch.setopt(pycurl.PROXYTYPE, 'HTTP')
-            ch.setopt(pycurl.PROXY, self.proxy)
-
-        if not self.proxyUser is None:
-            ch.setopt(pycurl.PROXYUSERPWD, self.proxyUser + ':' + self.proxyPass)
+        if self.parent.proxy:
+            ch.setopt(pycurl.PROXY, self.parent.proxyHost)
+            if self.parent.proxyAuth:
+                ch.setopt(pycurl.PROXYUSERPWD, self.parent.proxyAuth)
 
         ch.perform()
         resp = buffer.getvalue()
@@ -514,16 +510,8 @@ class HttpInterface:
 
         return body
 
-    def setProxy(self, proxy):
-        self.proxy = proxy
+    def verifyPeer(self, enable):
+        self.verifyPeer = enable
 
-    def setProxyPort(self, port):
-        self.proxyPort = port
-
-    def setProxyCredentials(self, username, password):
-        self.proxyUser = username
-        self.proxyPass = password
-
-    def clearProxy(self):
-        self.proxy = None
-        self.proxyUser = None
+    def verifyHost(self, enable):
+        self.verifyHost = enable
