@@ -1,6 +1,8 @@
 class User(object):
     def __init__(self, userData):
         self.username = None
+        self.has_anonymous_profile_picture = False
+        self.is_favorite = False
         self.profile_pic_url = None
         self.full_name = None
         self.pk = None
@@ -13,6 +15,10 @@ class User(object):
         self.pk = userData['pk']
         self.is_verified = userData['is_verified']
         self.is_private = userData['is_private']
+        if 'has_anonymous_profile_picture' in userData:
+            self.has_anonymous_profile_picture = userData['has_anonymous_profile_picture']
+        if 'is_favorite' in userData:
+            self.is_favorite = userData['is_favorite']
 
     def getUsername(self):
         return self.username
@@ -31,3 +37,9 @@ class User(object):
 
     def isPrivate(self):
         return self.is_private
+
+    def hasAnonymousProfilePicture(self):
+        return self.has_anonymous_profile_picture
+
+    def isFavorite(self):
+        return self.is_favorite
